<?php

/**
 * SquirrelMail LDAP Personal Address Book Backend Plugin (ldap_abook_backend)
 * By Daniel Marczisovszky <marczi@dev-labs.com>
 *
 * Based on Address book backend template by Tomas Kuliavas
 *
 * Copyright (c) 2005-2006 Daniel Marczisovszky <marczi@dev-labs.com>
 *
 * Licensed under the GNU GPL. For full terms see the file COPYING that came
 * with the Squirrelmail distribution.
 *
 * Version: 0.2.2
 */

if (!defined("SM_PATH")) {
    define("SM_PATH", "../../");
}

include_once(SM_PATH . "functions/addressbook.php");
include_once(SM_PATH . "functions/i18n.php");
include_once(SM_PATH . "functions/display_messages.php");

if (file_exists(SM_PATH . "plugins/ldap_abook_backend/config.php")) {
    require_once(SM_PATH . "plugins/ldap_abook_backend/config.php");
}
else {
    error_box("Configuration file for LDAP Personal Address Book Plugin is missing!");
}

if (file_exists(SM_PATH . "plugins/ldap_common/ldap_common.php")) {
    include_once(SM_PATH . "plugins/ldap_common/ldap_common.php");
}
else {
    error_box("LDAP Common Plugin is missing! Please download and install it!");
}

class abook_LdapBackend extends addressbook_backend {
    var $btype = 'local';
    var $bname = 'LdapBackend';
    
    var $writeable = true;

    var $ldap;

    var $host;
    var $options;
    var $useTls;
    var $useSelfAuth;

    var $bindDn;
    var $bindPassword;

    var $base;
    var $filter;

    // attribute names
    var $squirrelAttrs;
    var $ldapAttrs;
    var $requiredAttrs;
    var $requiredMsgs;

    var $attrNickname;
    var $objectClass;

    var $containerObjectClass;
    var $containerDnAttr;

    var $missingField;

    function abook_LdapBackend($param)
    {
	global $ldapAbookHost, $ldapAbookOptions, $ldapAbookUseTls,
    	       $ldapAbookUseSelfAuth, $ldapAbookBindDn, $ldapAbookBindPassword,
    	       $ldapAbookBase, $ldapAbookFilter,
	       $ldapAbookObjectClass,

    	       $ldapAbookRequiredNickname, $ldapAboobRequiredFirstname, $ldapAbookRequiredLastname,
               $ldapAbookRequiredEmail, $ldapAbookRequiredLabel,

    	       $ldapAbookAttrNickname, $ldapAbookAttrFirstname, $ldapAbookAttrLastname,
               $ldapAbookAttrEmail,$ldapAbookAttrLabel,

	       $ldapAbookContainerObjectClass, $ldapAbookContainerDnAttr,

	       $username, $default_charset, $languages, $squirrelmail_language;
	
					    
        if (!is_array($param)) {
            return $this->set_error('Invalid argument to constructor');
	}

        if (!empty($param['name'])) {
           $this->sname = $param['name'];
        }

        $charset = $default_charset;
        if (isset($languages[$squirrelmail_language])) {
    	    $charset = $languages[$squirrelmail_language]["CHARSET"];
        }
	$this->ldap = new LdapClient($charset);

        $this->host = $ldapAbookHost;
        $this->options = $ldapAbookOptions;
        $this->useTls = $ldapAbookUseTls;

        $this->base = $ldapAbookBase;
        $this->filter = $ldapAbookFilter;

        $this->useSelfAuth = $ldapAbookUseSelfAuth;
	if (!$this->useSelfAuth) {
            $this->bindDn = $ldapAbookBindDn;
	    $this->bindPassword = $ldapAbookBindPassword;
	}
	// Bind only if not in self auth mode
	if (!$this->connect(!$this->useSelfAuth)) {
	    $this->disconnect($this->_("Can't connect to LDAP server!"));
	}

    	$query = str_replace("%username", $username, $this->filter);
    	$this->debug("abook_LdapBackend::abook_LdapBackend - search user DN: $this->base QUERY: $query");
	$dn = $this->ldap->searchDn($this->base, $query, true, true);
	if (!$dn) {
	    $this->disconnect($this->_("Can't search in LDAP server!"));
	}
	$this->ldap->disconnect();
    	$this->debug("abook_LdapBackend::abook_LdapBackend - found user DN: $dn");

	// In self auth mode, the found DN will be used both for binding and as the base DN for the
	// personal address book.
	if ($this->useSelfAuth) {
	    // Fix by Patrick Scharrenberg
	    sqgetGlobalVar("key", $key, SQ_COOKIE);
	    sqgetGlobalVar("onetimepad", $onetimepad, SQ_SESSION);

    	    $this->bindDn = $dn;
	    $this->bindPassword = OneTimePadDecrypt($key, $onetimepad);
	}

	$this->base = $dn;
	$this->objectClass = $ldapAbookObjectClass;
	$this->containerDnAttr = $ldapAbookContainerDnAttr;
	$this->containerObjectClass = $ldapAbookContainerObjectClass;

	$this->attrNickname = $ldapAbookAttrNickname;
	$this->ldapAttrs = array(
	    $ldapAbookAttrNickname,
	    $ldapAbookAttrFirstname,
	    $ldapAbookAttrLastname,
	    $ldapAbookAttrEmail,
	    $ldapAbookAttrLabel);
	$this->requiredAttrs = array(
    	    $ldapAbookRequiredNickname, 
	    $ldapAboobRequiredFirstname, 
	    $ldapAbookRequiredLastname,
            $ldapAbookRequiredEmail,
	    $ldapAbookRequiredLabel);
	$this->squirrelAttrs = array(
	    "nickname", 
	    "firstname", 
	    "lastname", 
	    "email", 
	    "label");
	$this->requiredMsgs = array(
	    $this->_("Nickname is missing"),
	    $this->_("First name is missing"),
	    $this->_("Last name is missing"),
	    $this->_("Email is missing"),
	    $this->_("Additional info is missing"));

    	$this->debug("abook_LdapBackend::abook_LdapBackend - address book base: $this->base");
    }

    function _($s)
    {
        bindtextdomain("ldap_abook_backend", SM_PATH . "plugins/ldap_abook_backend/locale");
        textdomain("ldap_abook_backend");
        $s = _($s);
        bindtextdomain("squirrelmail", SM_PATH . "locale");
        textdomain("squirrelmail");
	return $s;
    }

    function debug($s)
    {
	global $ldapAbookDebug;
	if ($ldapAbookDebug) {
	    $this->ldap->writeLog($s);
	}
    }

    function connect($doBind = true)
    {
        $this->debug("abook_LdapBackend::connect - HOST: $this->host");
        if (!$this->ldap->connect($this->host, $this->options, $this->useTls)) {
            $this->disconnect($this->_("Can't connect to LDAP server!"));
        }

	if ($doBind) {
    	    $this->debug("abook_LdapBackend::connect - bind as USER: $this->bindDn PASSWORD: $this->bindPassword SELFAUTH: $this->useSelfAuth");
    	    if (!$this->ldap->bind($this->bindDn, $this->bindPassword)) {
        	$this->disconnect($this->_("Can't login to LDAP server!"));
    	    }
	}

	return true;
    }

    function disconnect($errorText = "")
    {
	$this->ldap->disconnect();
	if ($errorText != "" || $this->ldap->failed) {
	    return $this->set_error($errorText);
	}
	return true;
    }

    function getBase()
    {
	$base = $this->base;
	if (!empty($this->containerDnAttr)) {
	    $base = $this->containerDnAttr . "," . $base;
	}
	return $base;
    }

    function convertToLdap($userdata, $modification)
    {
	global $ldapAbookValidateEmail;

	if ($ldapAbookValidateEmail) {
	    $s = $userdata["email"];
	    $len = strlen($s);
	    for ($i = 0; $i < $len; $i++) {
		if (ord($s[$i]) > 127) {
		    $this->missingField = $this->_("Invalid character in email!");
		    return false;
		}
	    }
	}

	$object = array();
	for ($i = 0; $i < count($this->ldapAttrs); $i++) {
	    $s = $userdata[$this->squirrelAttrs[$i]];
	    if ($s != "") {
		$object[$this->ldapAttrs[$i]] = $s;
	    }
	    else if ($this->requiredAttrs[$i]) {
		$this->missingField = $this->requiredMsgs[$i];
		return false;
	    }
	    else if ($modification) {
		$object[$this->ldapAttrs[$i]] = array();
	    }
	}
	return $object;
    }

    /**
     * Search address function
     * @param expr string search expression
     */
    function search($expr, $exactSearch = false)
    {
	global $ldapAbookFullnameTemplate, $default_charset;

        /* To be replaced by advanded search expression parsing */
        if(is_array($expr)) {
	    return false;
	}

	if ($expr != "*") {
	    $expr = $this->ldap->escapeQuery($expr);
	}

        /* Encode the expression */
	if (!$exactSearch) {
    	    if(strstr($expr, '*') === false) {
        	$expr = "*$expr*";
    	    }
	}
        $query = "$this->attrNickname=$expr";

        // Make sure connection is there
        if(!$this->connect()) {
            return false;
        }

	$base = $this->getBase();
        $this->debug("abook_LdapBackend::search - search entries in DN: $base QUERY: $query");
        $objects = $this->ldap->searchObjects($base, $query, $this->ldapAttrs);

	$cnt = $objects["count"];
        $ret = array();
        if ($cnt == 0) {
	    $this->disconnect();
            return $ret;
        }

	// Convert LDAP result objects into SquirrelMail Address Book objects
        for($i = 0 ; $i < $cnt ; $i++) {
	    $object = array();
	    for ($j = 0; $j < count($this->ldapAttrs); $j++) {
		$key = $this->ldapAttrs[$j];
		if (isset($objects[$i][$key][0])) {
		    $s = $objects[$i][$key][0];
		}
		$object[$this->squirrelAttrs[$j]] = (!empty($s) ? $s : "");
	    }

	    $s = $ldapAbookFullnameTemplate;
	    $s = str_replace("%firstname", $object["firstname"], $s);
	    $s = str_replace("%lastname", $object["lastname"], $s);
	    $object["name"] = $s;

	    $object["backend"] = $this->bnum;
	    $object["source"] = &$this->sname;

            array_push($ret, $object);
        }

	$this->disconnect();

        return $ret;
    }
     
    /**
     * Lookup alias
     * @param alias string
     */
    function lookup($alias) {
        if (empty($alias)) {
            return array();
        }
         
        $alias = strtolower($alias);
	$objects = $this->search($alias, true);
	return $objects[0];
    }

    /**
     * List all addresses
     * @return array
     */
    function list_addr()
    {
        return $this->search("*");
    }

    /**
     * Add address
     * @param userdata
     * @return boolean
     */
    function add($userdata)
    {
	global $ldapAbookObjectClass;

        // See if user exist already
	$this->connect();

	$base = $this->getBase();
        $this->debug("abook_LdapBackend::add - check if alias exists: " . $userdata["nickname"]);
	$dn = $this->ldap->searchDn($base, 
				    $this->attrNickname . "=" . $userdata["nickname"], 
		                    false, false);
        if (!empty($dn)) {
    	    $this->debug("abook_LdapBackend::add - already exists");
	    return $this->set_error(sprintf($this->_("User '%s' already exist"), $userdata["nickname"]));
        }

	// If container object is used, check if it should be created first
	if (!empty($this->containerDnAttr)) {
	    if (!$this->ldap->existsDn($base)) {
		$containerAttr = explode("=", $this->containerDnAttr);
		$object = array();
		$object["objectClass"] = $this->containerObjectClass;
		$object[$containerAttr[0]] = $containerAttr[1];
		if (!$this->ldap->addObject($base, $object)) {
		    return $this->disconnect($this->_("Address add operation failed"));
		}
    	    }
	}

	$this->debug("convert");
	$object = $this->convertToLdap($userdata, false);
	if (!$object) {
	    return $this->disconnect($this->missingField);
	}
	$object["objectClass"] = $this->objectClass;
	$dn = $this->attrNickname . "=" . $this->ldap->escapeDn($userdata["nickname"]) . "," . $base;
        $this->debug("abook_LdapBackend::add - DN: $dn");
	if (!$this->ldap->addObject($dn, $object)) {
	    return $this->disconnect($this->_("Address add operation failed"));
	}

	return $this->disconnect();
    }

    /**
     * Modify address
     * @param alias
     * @param userdata
     * @return boolean
     */
    function modify($alias, $userdata)
    {
	global $ldapAbookObjectClass;

         /* See if user exist */
	$this->connect();

	$base = $this->getBase();
	$dn = $this->ldap->searchDn($base, "$this->attrNickname=$alias", false, true);
        if (empty($dn)) {
            return $this->set_error(sprintf($this->_("User '%s' does not exist"), $alias));
        }

	$dn = $this->attrNickname . "=" . $this->ldap->escapeDn($userdata["nickname"]) . "," . $base;

	$this->debug("abook_LdapBackend::modify - DN: $dn");
	// If nickname has not changed, a single modification is enough.
	if ($alias == $userdata["nickname"]) {
	    $object = $this->convertToLdap($userdata, true);
	    if (!$object) {
		return $this->disconnect($this->missingField);
	    }
	    // Remove the nickname attribute, it should not be modified.
	    unset($object[$this->attrNickname]);
	    if (!$this->ldap->modifyObject($dn, $object)) {
		return $this->disconnect($this->_("Address modify operation failed"));
	    }
	}
	// otherwise it should be recreated
	else {
	    $oldDn = $this->attrNickname . "=" . $this->ldap->escapeDn($alias) . "," . $base;
	    $object = $this->convertToLdap($userdata, true);
	    if (!$object) {
		return $this->disconnect($this->missingField);
	    }

	    $current = $this->ldap->readObject($oldDn);
	    $current = $this->ldap->convertOutputObjectToInputObject($current);
	    foreach ($object as $key => $value) {
		if (is_array($value) && count($value) == 0) {
		    unset($current[$key]);
		}
		else {
		    $current[$key] = $value;
		}
	    }

	    // Create the new one
	    if (!$this->ldap->addObject($dn, $current)) {
		return $this->disconnect($this->_("Address modify operation failed"));
	    }

	    // Delete the old one
	    if (!$this->ldap->deleteObject($oldDn)) {
		return $this->disconnect($this->_("Address modify operation failed"));
	    }
	}
	return $this->disconnect();
    }

    /**
     * Delete address
     * @param alias
     * @return boolean
     */
    function remove($aliases) {

	$this->connect();

	$base = $this->getBase();
	for ($i = 0; $i < count($aliases); $i++) {
	    $dn = $this->ldap->searchDn($base, "$this->attrNickname=$aliases[$i]", false, true);
	    $this->debug("abook_LdapBackend::remove - DN: $dn");
	    if ($dn == "" || !$this->ldap->deleteObject($dn)) {
		$this->debug("abook_LdapBackend::remove - failure");
		return $this->disconnect($this->_("Address delete operation failed"));
	    }
	}
	return $this->disconnect();
    }
}


?>